@@ -2,7 +2,7 @@
 FatRat download manager
 http://fatrat.dolezel.info
 
-Copyright (C) 2006-2008 Lubos Dolezel <lubos a dolezel.info>
+Copyright (C) 2006-2010 Lubos Dolezel <lubos a dolezel.info>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -38,6 +38,8 @@ respects for all of the code used other than "OpenSSL".
 #include <QProcess>
 #include <QSettings>
 #include <QUrl>
+#include <QApplication>
+#include <QClipboard>
 
 #include "MainWindow.h"
 #include "QueueDlg.h"
@@ -59,6 +61,8 @@ respects for all of the code used other than "OpenSSL".
 #include "AppTools.h"
 #include "AboutDlg.h"
 #include "ReportBugDlg.h"
+#include "filterlineedit.h"
+#include "ClipboardMonitor.h"
 
 #ifdef WITH_DOCUMENTATION
 #	include "tools/HelpBrowser.h"
@@ -98,6 +102,8 @@ MainWindow::MainWindow(bool bStartHidden)
 	}
 	
 	updateUi();
+
+	m_clipboardMonitor = new ClipboardMonitor;
 }
 
 MainWindow::~MainWindow()
@@ -105,10 +111,12 @@ MainWindow::~MainWindow()
 	saveWindowState();
 	
 	delete m_modelTransfers;
+	delete m_clipboardMonitor;
 }
 
 void MainWindow::setupUi()
 {
+	bool useSystemTheme;
 	Ui_MainWindow::setupUi(this);
 	
 	m_dropBox = new DropBox(this);
@@ -127,9 +135,15 @@ void MainWindow::setupUi()
 	
 	m_log = new LogManager(this, textTransferLog, textGlobalLog);
 	
+	useSystemTheme = getSettingsValue("gui/systemtheme").toBool();
 #ifdef WITH_DOCUMENTATION
 	QAction* action;
-	action = new QAction(QIcon(":/menu/about.png"), tr("Help"), menuHelp);
+	QIcon xicon;
+	if (useSystemTheme)
+		xicon = QIcon::fromTheme("help-contents", QIcon(":/menu/about.png"));
+	else
+		xicon = QIcon(":/menu/about.png");
+	action = new QAction(xicon, tr("Help"), menuHelp);
 	menuHelp->insertAction(actionAboutQt, action);
 	menuHelp->insertSeparator(actionAboutQt);
 	connect(action, SIGNAL(triggered()), this, SLOT(showHelp()));
@@ -153,7 +167,54 @@ void MainWindow::setupUi()
 		m_statusWidgets[i].first->show();
 	}
 
+	filterLineEdit = new Utils::FilterLineEdit(labelTransfers);
+	QSize ss = labelTransfers->size();
+
+	filterLineEdit->move(ss.width()-2-200,2);
+	filterLineEdit->resize(200, ss.height()-12);
+	connect(filterLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(filterTextChanged(const QString&)));
+	filterLineEdit->setFrame(false);
+	filterLineEdit->show();
+
 	fillSettingsMenu();
+	if (useSystemTheme)
+		applySystemTheme();
+}
+
+void MainWindow::resizeEvent(QResizeEvent* event)
+{
+	if (event)
+		QMainWindow::resizeEvent(event);
+	QSize ss = labelTransfers->size();
+	filterLineEdit->move(ss.width()-2-200,2);
+}
+
+void MainWindow::applySystemTheme()
+{
+	applySystemIcon(actionResume, "media-playback-start");
+	applySystemIcon(actionForcedResume, "media-seek-forward");
+	applySystemIcon(actionPause, "media-playback-pause");
+	applySystemIcon(actionTop, "go-top");
+	applySystemIcon(actionUp, "go-up");
+	applySystemIcon(actionDown, "go-down");
+	applySystemIcon(actionBottom, "go-bottom");
+	applySystemIcon(actionNewTransfer, "list-add");
+	applySystemIcon(actionDeleteTransfer, "edit-delete");
+	//applySystemIcon(actionDeleteTransferData, "edit-delete");
+	applySystemIcon(actionQuit, "application-exit");
+	applySystemIcon(actionProperties, "document-properties");
+	applySystemIcon(actionQueueProperties, "document-properties");
+	//applySystemIcon(actionNewQueue, "window-new");
+	//applySystemIcon(actionDeleteQueue, "window-close");
+	//applySystemIcon(actionRemoveCompleted, "tools-check-spelling");
+	applySystemIcon(actionSettings, "preferences-other");
+}
+
+void MainWindow::applySystemIcon(QAction* action, QString path)
+{
+	QIcon icon = QIcon::fromTheme(path);
+	if (!icon.isNull())
+		action->setIcon(icon);
 }
 
 void MainWindow::fillSettingsMenu()
@@ -235,6 +296,7 @@ void MainWindow::connectActions()
 	
 	connect(TransferNotifier::instance(), SIGNAL(stateChanged(Transfer*,Transfer::State,Transfer::State)), this, SLOT(downloadStateChanged(Transfer*,Transfer::State,Transfer::State)));
 	connect(actionBugReport, SIGNAL(triggered()), this, SLOT(reportBug()));
+	connect(actionCopyRemoteURI, SIGNAL(triggered()), this, SLOT(copyRemoteURI()));
 }
 
 void MainWindow::restoreWindowState(bool bStartHidden)
@@ -279,6 +341,7 @@ void MainWindow::restoreWindowState(bool bStartHidden)
 	
 	if(bStartHidden)
 		actionDisplay->setChecked(false);
+	resizeEvent(0);
 }
 
 void MainWindow::saveWindowState()
@@ -378,6 +441,8 @@ void MainWindow::updateUi()
 		actionNewTransfer->setEnabled(true);
 		actionStopAll->setEnabled(true);
 		actionResumeAll->setEnabled(true);
+
+		speedGraphQueue->setRenderSource(q);
 	}
 	else
 	{
@@ -391,6 +456,8 @@ void MainWindow::updateUi()
 	// transfer view
 	Transfer* d = 0;
 	int currentTab = tabMain->currentIndex();
+
+	bool hasFilter = !getFilterText().isEmpty();
 	
 	if(!sel.empty())
 	{
@@ -411,8 +478,8 @@ void MainWindow::updateUi()
 		
 		if(bSingle)
 		{
-			actionTop->setEnabled(sel[0]);
-			actionBottom->setEnabled(sel[0] < rcount-1);
+			actionTop->setEnabled(sel[0] && !hasFilter);
+			actionBottom->setEnabled(sel[0] < rcount-1 && !hasFilter);
 			
 			q->lock();
 			
@@ -435,14 +502,14 @@ void MainWindow::updateUi()
 			actionResume->setEnabled(true);
 			actionPause->setEnabled(true);
 			
-			actionTop->setEnabled(true);
-			actionUp->setEnabled(false);
-			actionDown->setEnabled(false);
-			actionBottom->setEnabled(true);
+			actionTop->setEnabled(!hasFilter);
+			//actionUp->setEnabled(false);
+			//actionDown->setEnabled(false);
+			actionBottom->setEnabled(!hasFilter);
 		}
 		
-		actionUp->setEnabled(true);
-		actionDown->setEnabled(true);
+		actionUp->setEnabled(!hasFilter);
+		actionDown->setEnabled(!hasFilter);
 		
 		actionDeleteTransfer->setEnabled(true);
 		actionDeleteTransferData->setEnabled(true);
@@ -478,7 +545,7 @@ void MainWindow::updateUi()
 		actionInfoBar->setEnabled(false);
 		actionProperties->setEnabled(false);
 		
-		speedGraph->setRenderSource(0);
+		speedGraph->setRenderSource((Queue*)NULL);
 		m_log->setLogSource(0);
 	}
 	
@@ -1143,13 +1210,13 @@ void MainWindow::transferOptions()
 {
 	WidgetHostDlg dlg(this);
 	
+	QList<int> sel = getSelection();
 	Queue* q = getCurrentQueue();
 	Transfer* d;
-	QModelIndex ctrans = treeTransfers->currentIndex();
 	
 	if(!q) return;
 	
-	d = q->at(ctrans.row());
+	d = q->at(sel[0]);
 	
 	if(d != 0)
 	{
@@ -1387,6 +1454,8 @@ void MainWindow::transferItemContext(const QPoint&)
 		menu.addAction(actionDown);
 		menu.addAction(actionBottom);
 		menu.addSeparator();
+		menu.addAction(actionCopyRemoteURI);
+		menu.addSeparator();
 		
 		Queue* q = getCurrentQueue();
 		if(q != 0)
@@ -1527,7 +1596,7 @@ QList<int> MainWindow::getSelection()
 	
 	if(Queue* q = getCurrentQueue())
 	{
-		int size = q->size();
+		int size = qMin(q->size(), m_modelTransfers->rowCount());
 		
 		doneQueue(q, true, false);
 		
@@ -1536,7 +1605,7 @@ QList<int> MainWindow::getSelection()
 			int row = in.row();
 			
 			if(row < size)
-				result << row;
+				result << m_modelTransfers->remapIndex(row);
 		}
 		
 		qSort(result);
@@ -1649,6 +1718,37 @@ void MainWindow::reportBug()
 	ReportBugDlg(this).exec();
 }
 
+void MainWindow::copyRemoteURI()
+{
+	Queue* q = getCurrentQueue();
+	QList<int> sel = getSelection();
+	QString uris;
+
+	if(!q) return;
+
+	foreach(int i,sel)
+	{
+		Transfer* t = q->at(i);
+		QString uri = t->remoteURI();
+
+		if (!uri.isEmpty())
+		{
+			if (!uris.isEmpty())
+				uris += '\n';
+			uris += uri;
+		}
+	}
+
+	doneQueue(q);
+	QApplication::clipboard()->setText(uris);
+}
+
+void MainWindow::filterTextChanged(const QString& text)
+{
+	treeTransfers->selectionModel()->clearSelection();
+	updateUi();
+}
+
 void addMenuAction(const MenuAction& action)
 {
 	m_menuActions << action;
