@@ -2,7 +2,7 @@
 FatRat download manager
 http://fatrat.dolezel.info
 
-Copyright (C) 2006-2008 Lubos Dolezel <lubos a dolezel.info>
+Copyright (C) 2006-2010 Lubos Dolezel <lubos a dolezel.info>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -26,6 +26,7 @@ respects for all of the code used other than "OpenSSL".
 */
 
 
+#include "config.h"
 #include "CurlDownload.h"
 #include "Settings.h"
 #include "RuntimeException.h"
@@ -43,6 +44,7 @@ respects for all of the code used other than "OpenSSL".
 #include <QMenu>
 #include <QtDebug>
 #include <iostream>
+#include <errno.h>
 
 CurlDownload::CurlDownload()
 	: m_curl(0), m_nTotal(0), m_nStart(0), m_bAutoName(false), m_nUrl(0), m_file(0)
@@ -131,9 +133,11 @@ int CurlDownload::acceptable(QString uri, bool)
 void CurlDownload::globalInit()
 {
 	new CurlPoller;
+
+	CurlPoller::setTransferTimeout(getSettingsValue("httpftp/timeout").toInt());
 	
 	SettingsItem si;
-	si.icon = QIcon(":/fatrat/httpftp.png");
+	si.icon = DelayedIcon(":/fatrat/httpftp.png");
 	si.title = tr("HTTP/FTP");
 	si.lpfnCreate = HttpFtpSettings::create;
 	
@@ -199,14 +203,14 @@ void CurlDownload::changeActive(bool bActive)
 			m_nUrl = 0;
 		
 		std::string spath = filePath().toStdString();
-		m_file = open(spath.c_str(), O_CREAT|O_RDWR, 0666);
-		if(!m_file)
+		m_file = open(spath.c_str(), O_CREAT|O_RDWR|O_LARGEFILE, 0666);
+		if(m_file < 0)
 		{
 			enterLogMessage(m_strMessage = strerror(errno));
 			setState(Failed);
 			return;
 		}
-		lseek(m_file, 0, SEEK_END);
+		lseek64(m_file, 0, SEEK_END);
 		
 		QByteArray ba, auth;
 		QUrl url = m_urls[m_nUrl].url;
@@ -262,7 +266,9 @@ void CurlDownload::changeActive(bool bActive)
 		curl_easy_setopt(m_curl, CURLOPT_USERAGENT, "FatRat/" VERSION);
 		curl_easy_setopt(m_curl, CURLOPT_USE_SSL, CURLUSESSL_TRY);
 		curl_easy_setopt(m_curl, CURLOPT_FTP_FILEMETHOD, CURLFTPMETHOD_SINGLECWD);
-		curl_easy_setopt(m_curl, CURLOPT_RESUME_FROM_LARGE, m_nStart = lseek(m_file, 0, SEEK_CUR));
+
+		m_nStart = lseek64(m_file, 0, SEEK_CUR);
+		curl_easy_setopt(m_curl, CURLOPT_RESUME_FROM_LARGE, m_nStart);
 
 		qDebug() << "Resume from" << m_nStart;
 
@@ -285,8 +291,9 @@ void CurlDownload::changeActive(bool bActive)
 		curl_easy_setopt(m_curl, CURLOPT_SSH_AUTH_TYPES, CURLSSH_AUTH_PASSWORD | CURLSSH_AUTH_KEYBOARD);
 		curl_easy_setopt(m_curl, CURLOPT_USE_SSL, CURLUSESSL_NONE);
 		
-		curl_easy_setopt(m_curl, CURLOPT_FTP_RESPONSE_TIMEOUT, 10);
-		curl_easy_setopt(m_curl, CURLOPT_CONNECTTIMEOUT, 10);
+		int timeout = getSettingsValue("httpftp/timeout").toInt();
+		curl_easy_setopt(m_curl, CURLOPT_FTP_RESPONSE_TIMEOUT, timeout);
+		curl_easy_setopt(m_curl, CURLOPT_CONNECTTIMEOUT, timeout);
 		
 		curl_easy_setopt(m_curl, CURLOPT_SEEKFUNCTION, seek_function);
 		curl_easy_setopt(m_curl, CURLOPT_SEEKDATA, &m_file);
@@ -305,10 +312,9 @@ void CurlDownload::changeActive(bool bActive)
 	{
 		resetStatistics();
 		CurlPoller::instance()->removeTransfer(this);
-		curl_easy_cleanup(m_curl);
 		m_curl = 0;
 		m_nStart = 0;
-		qDebug() << "Closing at pos" << lseek(m_file, 0, SEEK_CUR);
+		qDebug() << "Closing at pos" << lseek64(m_file, 0, SEEK_CUR);
 		close(m_file);
 		m_file = 0;
 	}
@@ -319,11 +325,10 @@ bool CurlDownload::writeData(const char* buffer, size_t bytes)
 	if(!isActive())
 		return false;
 	
-	if(m_curl /**&& (!m_nTotal || m_nTotal == -1LL || lseek(m_file, 0, SEEK_CUR) == 0)*/)
+	if(CURL* c = m_curl)
 	{
 		double len;
-		curl_easy_getinfo(m_curl, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &len);
-		//std::cout << "total = " << m_nStart << " + " << qlonglong(len) << "(was" << m_nTotal << ")";
+		curl_easy_getinfo(c, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &len);
 		if(len != -1 && len != 0)
 			m_nTotal = m_nStart + len;
 	}
@@ -331,6 +336,7 @@ bool CurlDownload::writeData(const char* buffer, size_t bytes)
 	if (write(m_file, buffer, bytes) != bytes)
 	{
 		m_strMessage = tr("Write failed");
+		enterLogMessage(tr("Write failed (%1)").arg(strerror(errno)));
 		return false;
 	}
 	
@@ -460,7 +466,7 @@ qulonglong CurlDownload::done() const
 		}
 	}
 	else
-		return lseek(m_file, 0, SEEK_CUR);
+		return lseek64(m_file, 0, SEEK_CUR);
 }
 
 void CurlDownload::load(const QDomNode& map)
@@ -587,3 +593,13 @@ WidgetHostChild* CurlDownload::createOptionsWidget(QWidget* w)
 	return new HttpOptsWidget(w, this);
 }
 
+QString CurlDownload::remoteURI() const
+{
+	if (m_urls.isEmpty())
+		return QString();
+	if (m_nUrl >= m_urls.size())
+		m_nUrl = 0;
+	QUrl url = m_urls[m_nUrl].url;
+	url.setUserInfo(QString());
+	return url.toString();
+}
