@@ -85,7 +85,7 @@ HttpService::HttpService()
 	SettingsItem si;
 	si.lpfnCreate = SettingsWebForm::create;
 	si.title = tr("Web Interface");
-	si.icon = QIcon(":/fatrat/webinterface.png");
+	si.icon = DelayedIcon(":/fatrat/webinterface.png");
 	
 	addSettingsPage(si);
 }
@@ -465,7 +465,7 @@ QByteArray HttpService::graph(QString queryString)
 	if(!q || !t)
 		return QByteArray();
 	
-	SpeedGraph::draw(t, QSize(640, 480), &image);
+	SpeedGraph::draw(t->speedData(), QSize(640, 480), &image);
 	
 	q->unlock();
 	g_queuesLock.unlock();
